/*
  matrix.c version 1.1 

  This program will generate a system of equations in matrix form with
  conditions that x, y and z will be integers in the range [-n, n].
  We'll guarantee the solution consists of integers by generating x, y
  and z at random and then genrating random coefficients.

  Michael J Chappell  April 18, 2022
*/

#include <stdio.h>
#include <time.h>
#include <stdlib.h>
#include <ctype.h>

#define DEFAULT_N 10

int main(int argc, char **argv) {
  int a[4][4];
  int i, j, n=DEFAULT_N;
  char again[5];

  /* 
     We're going to cheat a bit here and assume if we receive a
     parameter that it will be an integer greater than zero.
  */

  if (argc>1) {
    n=atoi(argv[1]);
  }
  if (n<1) {
    fprintf(stderr,
	    "usage: matrix [n]\n"
	    "n must be an integer greater than 0.\n"
	    "Default is %d\n", DEFAULT_N);
    return EXIT_FAILURE;
  }

  /* console is bugged on msys - turn buffering off */
  setvbuf(stdout, NULL, _IONBF, 0);
  setvbuf(stderr, NULL, _IONBF, 0);
  
  srand(time(NULL));

  /* a[0][1..3] will be x, y, z;
     subsequent rows will be coefficients;
     a[][0] will be the computed sum for that row.
  */
  do {
    for (j=0; j<4; j++) {
      for (i=0; i<4; i++) {
	a[j][i]=rand()%(2*n+1)-n;
      }
    }
    a[0][0]=0; /* value is unused */
    for (j=1; j<4; j++) {
      a[j][0]=0;
      for (i=1; i<4; i++) {
	a[j][0]+=a[0][i]*a[j][i];
      }
    }
    /* print rows 1..3 of the matrix */
    for (j=1; j<4; j++) {
      for (i=1; i<4; i++) {
	printf("% 3d ",a[j][i]);
      }
      printf(" | % 4d\n",a[j][0]);
    }
    printf("\nHit Enter for the solution\n");

    getchar();
    for (i=1; i<4; i++) {
      printf("% 3d ",a[0][i]);
    }
    printf("\n");

    printf("Another problem? ");
    fgets(again, 4, stdin);
  } while (tolower(again[0])=='y');

  return EXIT_SUCCESS;
}
